local Layer = CCLayer

function Layer:onTouchBegan(x, y)
    return true
end

function Layer:onTouchMoved(x, y)
end

function Layer:onTouchEnded(x, y)
end

function Layer:onTouchCancelled(x, y)
end

function Layer:onTouchesBegan( points )
    return true
end

function Layer:onTouchesMoved( points )
end

function Layer:onTouchesEnded( points )
end

function Layer:onTouchesCancelled( points )
end

function Layer:onTouch( isMultiTouches, touchPriority, swallowTouches )
    if type(isMultiTouches) ~= "boolean" then isMultiTouches = false end
    if type(touchPriority) ~= "number" then touchPriority = -1000 end
    if type(swallowTouches) ~= "boolean" then swallowTouches = false end
    
    local callbacks = {}
    if not isMultiTouches then
        callbacks.began = handler(self, self.onTouchBegan)
        callbacks.moved = handler(self, self.onTouchMoved)
        callbacks.ended = handler(self, self.onTouchEnded)
        callbacks.cancelled = handler(self, self.onTouchCancelled)
    else
        callbacks.began = handler(self, self.onTouchesBegan)
        callbacks.moved = handler(self, self.onTouchesMoved)
        callbacks.ended = handler(self, self.onTouchesEnded)
        callbacks.cancelled = handler(self, self.onTouchesCancelled)
    end

    self:registerScriptTouchHandler(function(state, ...)
        local args = {...}
        local callback = callbacks[state]
        if isMultiTouches then
            args = args[1]
            local points = {}
            for i = 1, #args, 3 do
                local x, y, id = args[i], args[i + 1], args[i + 2]
                points[id] = {x = x, y = y, id = id}
            end
            return callback(points)
        else
            return callback(args[1],args[2])
        end
    end, isMultiTouches, touchPriority, swallowTouches)
    self:setTouchEnabled(true)
    return self
end

function Layer:removeTouch()
    self:unregisterScriptTouchHandler()
    self:setTouchEnabled(false)
    return self
end

function Layer:onKeypad(callback)
    self:registerScriptKeypadHandler(callback)
    self:setKeyboardEnabled(true)
    return self
end

function Layer:removeKeypad()
    self:unregisterScriptKeypadHandler()
    self:setKeyboardEnabled(false)
    return self
end

function Layer:onAccelerate(callback)
    self:registerScriptAccelerateHandler(callback)
    self:setAccelerometerEnabled(true)
    return self
end

function Layer:removeAccelerate()
    self:unregisterScriptAccelerateHandler()
    self:setAccelerometerEnabled(false)
    return self
end
