local Sprite = CCSprite

function Sprite:playAnimationOnce(animation, args)
    local actions = {}

    local showDelay = args.showDelay or 0
    if showDelay then
        self:setVisible(false)
        actions[#actions + 1] = CCDelayTime:create(showDelay)
        actions[#actions + 1] = CCShow:create()
    end

    local delay = args.delay or 0
    if delay > 0 then
        actions[#actions + 1] = CCDelayTime:create(delay)
    end

    actions[#actions + 1] = CCAnimate:create(animation)

    if args.removeSelf then
        actions[#actions + 1] = CCRemoveSelf:create()
    end

    if args.onComplete then
        actions[#actions + 1] = CCCallFunc:create(args.onComplete)
    end

    local action
    if #actions > 1 then
        action = CCSequence:create(actions)
    else
        action = actions[1]
    end
    self:runAction(action)
    return action
end

function Sprite:playAnimationForever(animation)
    local animate = CCAnimate:create(animation)
    local action = CCRepeatForever:create(animate)
    self:runAction(action)
    return action
end

local _setBlendFunc = Sprite.setBlendFunc
function Sprite:setBlendFunc( src, dst )
    if dst then
        local t = ccBlendFunc:new()
        t.src = src
        t.dst = dst
        _setBlendFunc(self, t)
    else
        if type(src) == "table" then
            local t = ccBlendFunc:new()
            t.src = src.src
            t.dst = src.dst
            _setBlendFunc(self, t)
        else
            _setBlendFunc(self, src)
        end
    end
end