--
-- Author: Small_Lin
-- Date: 2017-05-24 14:48:21
--
local _M = class("UILayoutComponent", CCComponent)
local Node = CCNode
local __LAYOUT_COMPONENT_NAME = "__ui_layout"
local _activeLayout = true

local HorizontalEdge = {
    None = 0,
    Left = 1,
    Right = 2,
    Center = 3,
}

local VerticalEdge = {
    None = 0,
    Bottom = 1,
    Top = 2,
    Center = 3,
}

function _M:ctor()
	self:setName(__LAYOUT_COMPONENT_NAME)
	self._horizontalEdge = HorizontalEdge.None
	self._verticalEdge = VerticalEdge.None
	self._leftMargin = 0
	self._rightMargin = 0
	self._bottomMargin = 0
	self._topMargin = 0
	self._usingPositionPercentX = false
	self._positionPercentX = 0
	self._usingPositionPercentY = false
	self._positionPercentY = 0
	self._usingStretchWidth = false
	self._usingStretchHeight = false
	self._percentWidth = 0
	self._usingPercentWidth = false
	self._percentHeight = 0
	self._usingPercentHeight = false
	self._actived = true
	self._isPercentOnly = false
end

local function doLayout(rootNode)
    if not _activeLayout then return end

    local childs = rootNode:getChildren()
    for i=1,childs do
        local node = childs:objectAtIndex(i-1)
        local com = node:getComponent(__LAYOUT_COMPONENT_NAME)
        local parent = node:getParent()
        if com and parent then
            _M.refreshLayout(layoutComponent)
        end
    end
end

function _M:bindLayoutComponent( node )
	local layout = node:getComponent(__LAYOUT_COMPONENT_NAME)
	if layout then return layout end
	layout = _M:create()
	node:addComponent(layout)
	return layout
end

function _M:getOwnerParent()
	return self:getOwner():getParent()
end

function _M:refreshHorizontalMargin()
	local parent = self:getOwnerParent()
	if not parent then return end

	local _owner = self:getOwner()
	local ownerPoint = _owner:getPosition()
	local ownerAnchor = _owner:getAnchorPoint()
	local ownerSize = _owner:getContentSize()
	local parentSize = parent:getContentSize()

	self._bottomMargin = ownerPoint.y - ownerAnchor.y * ownerSize.height
	self._topMargin = parentSize.height - (ownerPoint.y + (1 - ownerAnchor.y) * ownerSize.height)
end

function _M:setUsingPercentContentSize(isUsed)
    self._usingPercentWidth = isUsed
    self._usingPercentHeight = isUsed
end

function _M:getUsingPercentContentSize()
    return self._usingPercentWidth and self._usingPercentHeight
end

function _M:setPercentContentSize(percent)
    self:setPercentWidth(percent.x)
    self:setPercentHeight(percent.y)
end

function _M:getAnchorPosition()
    return self:getOwner():getAnchorPoint()
end

function _M:setAnchorPosition(point)
	local _owner = self:getOwner()
    local oldRect = _owner:getBoundingBox()
    _owner:setAnchorPoint(point)
    local newRect = _owner:getBoundingBox()
    local offSetX = oldRect.origin.x - newRect.origin.x
    local offSetY = oldRect.origin.y - newRect.origin.y

    local ownerPosition = _owner:getPosition()
    ownerPosition.x = ownerPosition.x + offSetX
    ownerPosition.y = ownerPosition.y + offSetY

    self:setPosition(ownerPosition)
end

function _M:getPosition()
    return self:getOwner():getPosition()
end

function _M:setPosition(position)
    local parent = self:getOwnerParent()
    local _owner = self:getOwner()
    if parent then
        local ownerPoint = position
        local parentSize = parent:getContentSize()

        if parentSize.width ~= 0 then
            self._positionPercentX = ownerPoint.x / parentSize.width
        else
            self._positionPercentX = 0
            if self._usingPositionPercentX then
                ownerPoint.x = 0
            end
        end

        if parentSize.height ~= 0 then
            self._positionPercentY = ownerPoint.y / parentSize.height
        else
            self._positionPercentY = 0
            if self._usingPositionPercentY then
                ownerPoint.y = 0
            end
        end

        _owner:setPosition(ownerPoint)

        self:refreshHorizontalMargin()
        self:refreshVerticalMargin()
    else
        _owner:setPosition(position)
    end
end

function _M:isPositionPercentXEnabled()
    return self._usingPositionPercentX
end
function _M:setPositionPercentXEnabled(isUsed)
    self._usingPositionPercentX = isUsed
    if (self._usingPositionPercentX) then
        self._horizontalEdge = HorizontalEdge.None
    end
end

function _M:getPositionPercentX()
    return self._positionPercentX
end
function _M:setPositionPercentX(percentMargin)
    self._positionPercentX = percentMargin

    local parent = self:getOwnerParent()
    if (parent) then
        _owner:setPositionX(parent:getContentSize().width * self._positionPercentX)
        self:refreshHorizontalMargin()
    end
end

function _M:isPositionPercentYEnabled()
    return self._usingPositionPercentY
end
function _M:setPositionPercentYEnabled(isUsed)
    self._usingPositionPercentY = isUsed
    if (self._usingPositionPercentY) then
        self._verticalEdge = VerticalEdge.None
    end
end

function _M:getPositionPercentY()
    return self._positionPercentY
end
function _M:setPositionPercentY(percentMargin)
    self._positionPercentY = percentMargin

    local parent = self:getOwnerParent()
    local _owner = self:getOwner()
    if (parent) then
        _owner:setPositionY(parent:getContentSize().height * self._positionPercentY)
        self:refreshVerticalMargin()
    end
end

function _M:getHorizontalEdge()
    return self._horizontalEdge
end
function _M:setHorizontalEdge(hEage)
    self._horizontalEdge = hEage
    if (self._horizontalEdge ~= HorizontalEdge.None) then
        self._usingPositionPercentX = false
    end

    local parent = self:getOwnerParent()
    local _owner = self:getOwner()
    if (parent) then
        local ownerPoint = _owner:getPosition()
        local parentSize = parent:getContentSize()
        if (parentSize.width ~= 0) then
            self._positionPercentX = ownerPoint.x / parentSize.width
        else
            self._positionPercentX = 0
            ownerPoint.x = 0
            if (self._usingPositionPercentX) then
                _owner:setPosition(ownerPoint)
            end
        end

        self:refreshHorizontalMargin()
    end
end

function _M:getVerticalEdge()
    return self._verticalEdge
end
function _M:setVerticalEdge(vEage)
    self._verticalEdge = vEage
    if (self._verticalEdge ~= VerticalEdge.None) then
        self._usingPositionPercentY = false
    end

    local parent = self:getOwnerParent()
    local _owner = self:getOwner()
    if (parent) then
        local ownerPoint = _owner:getPosition()
        local parentSize = parent:getContentSize()
        if (parentSize.height ~= 0) then
            self._positionPercentY = ownerPoint.y / parentSize.height
        else
            self._positionPercentY = 0
            ownerPoint.y = 0
            if (self._usingPositionPercentY) then
                _owner:setPosition(ownerPoint)
            end
        end

        self:refreshVerticalMargin()
    end
end

function _M:getLeftMargin()
    return self._leftMargin
end
function _M:setLeftMargin(margin)
    self._leftMargin = margin
end

function _M:getRightMargin()
    return self._rightMargin
end
function _M:setRightMargin(margin)
    self._rightMargin = margin
end

function _M:getTopMargin()
    return self._topMargin
end
function _M:setTopMargin(margin)
    self._topMargin = margin
end

function _M:getBottomMargin()
    return self._bottomMargin
end
function _M:setBottomMargin(margin)
    self._bottomMargin = margin
end

function _M:getSize()
    return self:getOwner():getContentSize()
end
function _M:setSize(size)
    local parent = self:getOwnerParent()
    local _owner = self:getOwner()
    local setSize = _owner.setSize or _owner.setContentSize
    if (parent) then
        local ownerSize = size
        local parentSize = parent:getContentSize()

        if (parentSize.width ~= 0) then
            self._percentWidth = ownerSize.width / parentSize.width
        else
            self._percentWidth = 0
            if (self._usingPercentWidth) then
                ownerSize.width = 0
            end
        end

        if (parentSize.height ~= 0) then
            self._percentHeight = ownerSize.height / parentSize.height
        else
            self._percentHeight = 0
            if (self._usingPercentHeight) then
                ownerSize.height = 0
            end
        end

        print("UILayoutComponent setSize", ownerSize.width, ownerSize.height)
        setSize(_owner, ownerSize)

        self:refreshHorizontalMargin()
        self:refreshVerticalMargin()
    else
        print("UILayoutComponent setSize", size.width, size.height)
        setSize(_owner, size)
    end
end

function _M:isPercentWidthEnabled()
    return self._usingPercentWidth
end
function _M:setPercentWidthEnabled(isUsed)
    self._usingPercentWidth = isUsed
    if (self._usingPercentWidth) then
        self._usingStretchWidth = false
    end
end

function _M:getSizeWidth()
    return _owner:getContentSize().width
end
function _M:setSizeWidth(width)
    local ownerSize = _owner:getContentSize()
    ownerSize.width = width

    local parent = self:getOwnerParent()
    local _owner = self:getOwner()
    if (parent) then
        local parentSize = parent:getContentSize()
        if (parentSize.width ~= 0) then
            self._percentWidth = ownerSize.width / parentSize.width
        else
            self._percentWidth = 0
            if (self._usingPercentWidth) then
                ownerSize.width = 0
            end
        end
        _owner:setContentSize(ownerSize)
        self:refreshHorizontalMargin()
    else
        _owner:setContentSize(ownerSize)
    end
end

function _M:getPercentWidth()
    return self._percentWidth
end
function _M:setPercentWidth(percentWidth)
    self._percentWidth = percentWidth

    local parent = self:getOwnerParent()
    local _owner = self:getOwner()
    if (parent) then
        local ownerSize = _owner:getContentSize()
        ownerSize.width = parent:getContentSize().width * self._percentWidth
        _owner:setContentSize(ownerSize)

        self:refreshHorizontalMargin()
    end
end

function _M:isPercentHeightEnabled()
    return self._usingPercentHeight
end
function _M:setPercentHeightEnabled(isUsed)
    self._usingPercentHeight = isUsed
    if (self._usingPercentHeight) then
        self._usingStretchHeight = false
    end
end

function _M:getSizeHeight()
    return _owner:getContentSize().height
end
function _M:setSizeHeight(height)
    local ownerSize = _owner:getContentSize()
    ownerSize.height = height

    local parent = self:getOwnerParent()
    local _owner = self:getOwner()
    if (parent) then
        local parentSize = parent:getContentSize()
        if (parentSize.height ~= 0) then
            self._percentHeight = ownerSize.height / parentSize.height
        else
            self._percentHeight = 0
            if (self._usingPercentHeight) then
                ownerSize.height = 0
            end
        end
        _owner:setContentSize(ownerSize)
        self:refreshVerticalMargin()
    else
        _owner:setContentSize(ownerSize)
    end
end

function _M:getPercentHeight()
    return self._percentHeight
end
function _M:setPercentHeight(percentHeight)
    self._percentHeight = percentHeight

    local parent = self:getOwnerParent()
    local _owner = self:getOwner()
    if (parent) then
        local ownerSize = _owner:getContentSize()
        ownerSize.height = parent:getContentSize().height * self._percentHeight
        _owner:setContentSize(ownerSize)

        self:refreshVerticalMargin()
    end
end

function _M:isStretchWidthEnabled()
    return self._usingStretchWidth
end
function _M:setStretchWidthEnabled(isUsed)
    self._usingStretchWidth = isUsed
    if (self._usingStretchWidth) then
        self._usingPercentWidth = false
    end
end

function _M:isStretchHeightEnabled()
    return self._usingStretchHeight
end
function _M:setStretchHeightEnabled(isUsed)
    self._usingStretchHeight = isUsed
    if (self._usingStretchHeight) then
        self._usingPercentHeight = false
    end
end

function _M:refreshLayout()
    if (not self._actived) then return end
    
    local parent = self:getOwnerParent()
    local _owner = self:getOwner()
    if not parent then return end

    local parentSize = parent:getContentSize()
    local ownerAnchor = _owner:getAnchorPoint()
    local ownerSize = _owner:getContentSize()
    local ownerPosition = _owner:getPosition()

    if (self._horizontalEdge == HorizontalEdge.None) then
        if (self._usingStretchWidth and not self._isPercentOnly) then
            ownerSize.width = parentSize.width * self._percentWidth
            self.ownerPosition.x = _leftMargin + ownerAnchor.x * ownerSize.width
        else
            if (self._usingPositionPercentX) then
                ownerPosition.x = parentSize.width * self._positionPercentX
            end
            if (self._usingPercentWidth) then
                ownerSize.width = parentSize.width * self._percentWidth
            end
        end
    elseif (self._horizontalEdge == HorizontalEdge.Left) then
        if (not self._isPercentOnly) then
            if (self._usingPercentWidth or self._usingStretchWidth) then
                ownerSize.width = parentSize.width * self._percentWidth
            end
            self.ownerPosition.x = _leftMargin + ownerAnchor.x * ownerSize.width
        end
    elseif self._horizontalEdge == HorizontalEdge.Right then
        if (not self._isPercentOnly) then
            if (self._usingPercentWidth or self._usingStretchWidth) then
                ownerSize.width = parentSize.width * self._percentWidth
            end
            self.ownerPosition.x = parentSize.width - (_rightMargin + (1 - ownerAnchor.x) * ownerSize.width)
        end
    elseif self._horizontalEdge == HorizontalEdge.Center then
        if (not self._isPercentOnly) then
            if (self._usingStretchWidth) then
                self.self.ownerSize.width = parentSize.width - _leftMargin - _rightMargin
                if (ownerSize.width < 0) then
                    ownerSize.width = 0
                end
                self.ownerPosition.x = _leftMargin + ownerAnchor.x * ownerSize.width
            else
                if (self._usingPercentWidth) then
                    ownerSize.width = parentSize.width * self._percentWidth
                end
                ownerPosition.x = parentSize.width * self._positionPercentX
            end
        end
    else
    end

    if self._verticalEdge == VerticalEdge.None then
        if (self._usingStretchHeight and not self._isPercentOnly) then
            ownerSize.height = parentSize.height * self._percentHeight
            ownerPosition.y = self._bottomMargin + ownerAnchor.y * ownerSize.height
        else
            if (self._usingPositionPercentY) then
                ownerPosition.y = parentSize.height * self._positionPercentY
            end
            if (self._usingPercentHeight) then
                ownerSize.height = parentSize.height * self._percentHeight
            end
        end
    elseif self._verticalEdge == VerticalEdge.Bottom then
        if (not self._isPercentOnly) then
            if (self._usingPercentHeight or self._usingStretchHeight) then
                ownerSize.height = parentSize.height * self._percentHeight
            end
            ownerPosition.y = self._bottomMargin + ownerAnchor.y * ownerSize.height
        end
    elseif self._verticalEdge == VerticalEdge.Top then
        if (not self._isPercentOnly) then
            if (self._usingPercentHeight or self._usingStretchHeight) then
                ownerSize.height = parentSize.height * self._percentHeight
            end
            ownerPosition.y = parentSize.height - (self._topMargin + (1 - ownerAnchor.y) * ownerSize.height)
        end
    elseif self._verticalEdge == VerticalEdge.Center then
        if (not self._isPercentOnly) then
            if (self._usingStretchHeight) then
                ownerSize.height = parentSize.height - self._topMargin - self._bottomMargin
                if (ownerSize.height < 0) then
                    ownerSize.height = 0
                end
                ownerPosition.y = self._bottomMargin + ownerAnchor.y * ownerSize.height
            else
                if (self._usingPercentHeight) then
                    ownerSize.height = parentSize.height * self._percentHeight
                end
                ownerPosition.y = parentSize.height* self._positionPercentY
            end
        end
    else
    end

    _owner:setPosition(ownerPosition)
    _owner:setContentSize(ownerSize)

    if _owner.getPages then
        _owner:requestDoLayout()
        _owner:sortAllChildren()

        local array = page:getPages()
        for i=1,array:count() do
            doLayout(array:objectAtIndex(i-1))
        end
    else
        doLayout(_owner)
    end
end

function _M:setActiveEnabled(enable)
    self._actived = enable
end

function _M:setPercentOnlyEnabled(enable)
    self._isPercentOnly = enable
end

return _M